//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
// TITLE : CAN transmit thread unit - CANTransmit.pas
//
// DESCRIPTION : Thread to transmit CAN data.
//
// DATE : 21/10/13

// MODIFICATIONS :
//    04/10/11 Created
//    21/10/13 Gap time adjusted from us to ms
//
// AUTHOR : Martyn Brown
//
//  2011-13 Netronics Ltd. All rights reserved.
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
unit CANTransmit;

interface

uses
  Classes;

type
  CANTransmitThread = class(TThread)
  private
    { Private declarations }
  protected
    procedure Execute; override;
  end;

implementation

uses
  Main, CANdoImport, CRC, HPTimer;

{ 
  Important: Methods and properties of objects in visual components can only be
  used in a method called using Synchronize, for example,

      Synchronize(UpdateCaption);

  and UpdateCaption could look like,

    procedure Transmit.UpdateCaption;
    begin
      Form1.Caption := 'Updated in a thread';
    end; 
    
    or 
    
    Synchronize(
      procedure
      begin
        Form1.Caption := 'Updated in thread via an anonymous method'
      end
      )
    );

  where an anonymous method is passed.

  Similarly, the developer can call the Queue method with similar parameters as
  above, instead passing another TThread class as the first parameter, putting
  the calling thread in a queue with the other thread.

}

{ Transmit }

//--------------------------------------------------------------------------
procedure CANTransmitThread.Execute;
var
  Data : array [0..7] of Byte;
  FrameCount, TotalFrames, Gap : Integer;
  XSum : Word;
begin
  // TRANSMIT
  XSum := Form1.XSum;
  TotalFrames := Form1.TotalFrames;
  Gap := Form1.Gap.Value;  // Interframe gap in ms
  FrameCount := 0;
  repeat
    // Frame count
    Data[0] := FrameCount and $FF;
    Data[1] := (FrameCount shr 8) and $FF;
    Data[2] := (FrameCount shr 16) and $FF;
    Data[3] := (FrameCount shr 24) and $FF;
    // Check bytes
    Data[4] := $55;
    Data[5] := $AA;
    // XSum
    CalculateCRC(@Data, 6, XSum);
    Data[6] := XSum and $FF;
    Data[7] := (XSum shr 8) and $FF;
    // Transmit CAN frame
    if (CANdoTransmit(@Form1.CANdoUSB, CANDO_ID_29_BIT, IDD, CANDO_DATA_FRAME, 8, @Data, 0, 0) = CANDO_SUCCESS) then
      HPWait(Gap)  // Pause for inter-frame gap
    else
      Terminate;  // Terminate thread
    Inc(FrameCount);
  until (FrameCount = TotalFrames) or Terminated;

  Form1.XSum := XSum;
  Form1.FrameCount := FrameCount;
end;
//--------------------------------------------------------------------------
end.
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
