﻿//------------------------------------------------------------------------------
//  TITLE :- Settings form for CANdo OBD demo. - Settings.cs
//  AUTHOR :- Martyn Brown
//  DATE :- 16/01/12
//
//  DESCRIPTION :- CANdo OBD 'C#' demo. program settings.
//
//  UPDATES :-
//	16/01/12 Created
//
//  LICENSE :-
//  This demonstration software is provided for use with the CANdo device &
//  is issued as FREE software, meaning that it is free for personal,
//  educational & commercial use, without restriction or time limit. The
//  software maybe modified & derivative works created, provided that support
//  for the CANdo device is maintained within the software. The software is
//  supplied "as is", with no implied warranties or guarantees.
//
//  (c) 2012 Netronics Ltd. All rights reserved.
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CANdoCSharp;

namespace CANdoOBD
{
  public partial class Settings : Form
  {
    //-----------------------------------------------------------------------
    // Accessors.
    //-----------------------------------------------------------------------
    public uint GetBaudRate { get { return Values.BaudRate; } }
    public byte GetIDE { get { return Values.IDE; } }
    public uint GetTxFunctionalID { get { return Values.TxFunctionalID; } }
    public uint GetTxPhysicalID { get { return Values.TxPhysicalID; } }
    public uint GetRxID { get { return Values.RxID; } }
    public int GetUpdateRate { get { return Values.UpdateRate; } }

    private struct TSettings
    {
      public uint BaudRate;
      public byte IDE;
      public uint TxFunctionalID;
      public uint TxPhysicalID;
      public uint RxID;
      public int UpdateRate;
    }

    // Request IDs
    private const uint ID11BitFunctionalAddress = 0x7DF;  // ISO 15765 defined 11 bit functional request address ID
    private const uint ID11BitECU1PhysicalAddress = 0x7E0;  // ISO 15765 defined 11 bit ECU 1 request address ID
    private const uint ID11BitECU2PhysicalAddress = 0x7E1;  // ISO 15765 defined 11 bit ECU 2 request address ID
    private const uint ID11BitECU3PhysicalAddress = 0x7E2;  // ISO 15765 defined 11 bit ECU 3 request address ID
    private const uint ID29BitFunctionalAddress = 0x18DB33F1;  // ISO 15765 defined 29 bit functional request address ID
    private const uint ID29BitECU1PhysicalAddress = 0x18DA10F1;  // ISO 15765 defined 29 bit ECU 1 request address ID
    private const uint ID29BitECU2PhysicalAddress = 0x18DA18F1;  // ISO 15765 defined 29 bit ECU 2 request address ID
    private const uint ID29BitECU3PhysicalAddress = 0x18DA28F1;  // ISO 15765 defined 29 bit ECU 3 request address ID
    // Response IDs
    private const uint ID11BitECU1Address = 0x7E8;  // ISO 15765 defined 11 bit ECU 1 response address ID
    private const uint ID11BitECU2Address = 0x7E9;  // ISO 15765 defined 11 bit ECU 2 response address ID
    private const uint ID11BitECU3Address = 0x7EA;  // ISO 15765 defined 11 bit ECU 3 response address ID
    private const uint ID29BitECU1Address = 0x18DAF110;  // ISO 15765 defined 29 bit ECU 1 response address ID
    private const uint ID29BitECU2Address = 0x18DAF118;  // ISO 15765 defined 29 bit ECU 2 response address ID
    private const uint ID29BitECU3Address = 0x18DAF128;  // ISO 15765 defined 29 bit ECU 3 response address ID
    // Settings
    private TSettings Values;

    //-----------------------------------------------------------------------
    // Init. Settings variables.
    //-----------------------------------------------------------------------
    public Settings()
    {
      InitializeComponent();

      // Init. default settings
      Values.BaudRate = 500;
      Values.IDE = CANdoImport.ID_11_BIT;
      Values.TxFunctionalID = ID11BitFunctionalAddress;
      Values.TxPhysicalID = ID11BitECU1PhysicalAddress;
      Values.RxID = ID11BitECU1Address;
      Values.UpdateRate = 1;
    }

    //-----------------------------------------------------------------------
    // OK button pressed, so update Settings variables & then close dialog.
    //-----------------------------------------------------------------------
    private void button1_Click(object sender, EventArgs e)
    {
      // 'OK' button, so update values
      if (radioButton1.Checked)
        Values.BaudRate = 250;
      else
        Values.BaudRate = 500;
      if (radioButton3.Checked)
        Values.IDE = CANdoImport.ID_11_BIT;
      else
        Values.IDE = CANdoImport.ID_29_BIT;
      Values.TxFunctionalID = UInt32.Parse(textBox1.Text, System.Globalization.NumberStyles.HexNumber);
      Values.TxPhysicalID = UInt32.Parse(textBox3.Text, System.Globalization.NumberStyles.HexNumber);
      Values.RxID = UInt32.Parse(textBox2.Text, System.Globalization.NumberStyles.HexNumber);
      Values.UpdateRate = comboBox1.SelectedIndex;

      Close();
    }

    //-----------------------------------------------------------------------
    // Cancel button pressed, so close dialog.
    //-----------------------------------------------------------------------
    private void button2_Click(object sender, EventArgs e)
    {
      // 'Cancel' button
      Close();
    }

    //-----------------------------------------------------------------------
    // Settings dialog displayed, so display current Settings values.
    //-----------------------------------------------------------------------
    private void Settings_Shown(object sender, EventArgs e)
    {
      // Display current settings
      if (Values.BaudRate == 250)
        radioButton1.Checked = true;  // 250k
      else
        radioButton2.Checked = true;  // 500k
      if (Values.IDE == CANdoImport.ID_11_BIT)
      {
        // 11 bit
        radioButton3.Checked = true;
        if (Values.RxID == ID11BitECU3Address)
          radioButton7.Checked = true;
        else
        if (Values.RxID == ID11BitECU2Address)
          radioButton6.Checked = true;
        else
          radioButton5.Checked = true;
      }
      else
      {
        // 29 bit
        radioButton4.Checked = true;
        if (Values.RxID == ID29BitECU3Address)
          radioButton7.Checked = true;
        else
        if (Values.RxID == ID29BitECU2Address)
          radioButton6.Checked = true;
        else
          radioButton5.Checked = true;
      }
      textBox1.Text = Values.TxFunctionalID.ToString("X");
      textBox3.Text = Values.TxPhysicalID.ToString("X");
      textBox2.Text = Values.RxID.ToString("X");
      comboBox1.SelectedIndex = Values.UpdateRate;
    }

    private void radioButton3_CheckedChanged(object sender, EventArgs e)
    {
      // Tx ID
      UpdateTxID();
      // Rx ID
      UpdateRxID();
    }

    private void UpdateTxID()
    {
      // Tx ID
      if (radioButton3.Checked)
      {
        // 11 bit ID
        textBox1.Text = ID11BitFunctionalAddress.ToString("X");
        if (radioButton5.Checked)
          textBox3.Text = ID11BitECU1PhysicalAddress.ToString("X");
        else
        if (radioButton6.Checked)
          textBox3.Text = ID11BitECU2PhysicalAddress.ToString("X");
        else
          textBox3.Text = ID11BitECU3PhysicalAddress.ToString("X");
      }
      else
      {
        // 29 bit ID
        textBox1.Text = ID29BitFunctionalAddress.ToString("X");
        if (radioButton5.Checked)
          textBox3.Text = ID29BitECU1PhysicalAddress.ToString("X");
        else
        if (radioButton6.Checked)
          textBox3.Text = ID29BitECU2PhysicalAddress.ToString("X");
        else
          textBox3.Text = ID29BitECU3PhysicalAddress.ToString("X");
      }
    }

    private void UpdateRxID()
    {
      if (radioButton3.Checked)
      {
        // 11 bit ID
        if (radioButton5.Checked)
          // ECU 1
          textBox2.Text = ID11BitECU1Address.ToString("X");
        else
        if (radioButton6.Checked)
          // ECU 2
          textBox2.Text = ID11BitECU2Address.ToString("X");
        else
        if (radioButton7.Checked)
          // ECU 3
          textBox2.Text = ID11BitECU3Address.ToString("X");
      }
      else
      {
        // 29 bit ID
        if (radioButton5.Checked)
          // ECU 1
          textBox2.Text = ID29BitECU1Address.ToString("X");
        else
        if (radioButton6.Checked)
          // ECU 2
          textBox2.Text = ID29BitECU2Address.ToString("X");
        else
        if (radioButton7.Checked)
          // ECU 3
          textBox2.Text = ID29BitECU3Address.ToString("X");
      }
    }
  }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
